------------------------------------------------------------------------------
------------------------------------------------------------------------------
--                                                                          --
-- Copyright (c) 2006-2009 Tobias Gubener                                   -- 
-- Subdesign CPC T-REX by TobiFlex                                          --
--                                                                          --
-- This source file is free software: you can redistribute it and/or modify --
-- it under the terms of the GNU General Public License as published        --
-- by the Free Software Foundation, either version 3 of the License, or     --
-- (at your option) any later version.                                      --
--                                                                          --
-- This source file is distributed in the hope that it will be useful,      --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of           --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            --
-- GNU General Public License for more details.                             --
--                                                                          --
-- You should have received a copy of the GNU General Public License        --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.    --
--                                                                          --
------------------------------------------------------------------------------
------------------------------------------------------------------------------


LIBRARY ieee;
USE ieee.std_logic_1164.all; 

ENTITY Extender IS 
	PORT
	(
		clk_14 :  IN  STD_LOGIC;
		clk_28 :  IN  STD_LOGIC;
		addbi :  IN  STD_LOGIC_VECTOR(5 DOWNTO 0);
		DRAM_DQ :  INOUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
		mdb :  IN  STD_LOGIC_VECTOR(7 DOWNTO 0);
		nHSync :  OUT  STD_LOGIC;
		nVSync :  OUT  STD_LOGIC;
		sigma1 :  OUT  STD_LOGIC;
		sigma2 :  OUT  STD_LOGIC;
		DRAM_CLK :  OUT  STD_LOGIC;
		DRAM_WE_N :  OUT  STD_LOGIC;
		DRAM_CAS_N :  OUT  STD_LOGIC;
		DRAM_RAS_N :  OUT  STD_LOGIC;
		addbo :  OUT  STD_LOGIC_VECTOR(8 DOWNTO 0);
		dac :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
		DPM :  OUT  STD_LOGIC_VECTOR(1 DOWNTO 0);
		DRAM_ADDR :  OUT  STD_LOGIC_VECTOR(11 DOWNTO 0);
		DRAM_BA :  OUT  STD_LOGIC_VECTOR(1 DOWNTO 0)
	);
END Extender;

ARCHITECTURE bdf_type OF Extender IS 

COMPONENT sdram
	PORT(zena_in : IN STD_LOGIC;
		 sysclk : IN STD_LOGIC;
		 reset : IN STD_LOGIC;
		 zwr : IN STD_LOGIC;
		 wsgin : IN STD_LOGIC;
		 vrefresh : IN STD_LOGIC;
		 rmem : IN STD_LOGIC;
		 rwr : IN STD_LOGIC;
		 rrefresh : IN STD_LOGIC;
		 speed : IN STD_LOGIC;
		 resetin : IN STD_LOGIC;
		 shresetin : IN STD_LOGIC;
		 datawr : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 mdb : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 rAddr : IN STD_LOGIC_VECTOR(23 DOWNTO 0);
		 rstate : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
		 sdata : INOUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		 vAddr : IN STD_LOGIC_VECTOR(23 DOWNTO 0);
		 zAddr : IN STD_LOGIC_VECTOR(23 DOWNTO 0);
		 zdatawr : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 zstate : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
		 sd_we : OUT STD_LOGIC;
		 sd_ras : OUT STD_LOGIC;
		 sd_cas : OUT STD_LOGIC;
		 cpuclk : OUT STD_LOGIC;
		 rena_o : OUT STD_LOGIC;
		 zena_o : OUT STD_LOGIC;
		 zwait : OUT STD_LOGIC;
		 vena : OUT STD_LOGIC;
		 reset_out : OUT STD_LOGIC;
		 c1MHz : OUT STD_LOGIC;
		 IOB_clk : OUT STD_LOGIC;
		 cpcreset : OUT STD_LOGIC;
		 shreset : OUT STD_LOGIC;
		 wsg : OUT STD_LOGIC;
		 ba : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
		 dataout : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		 dqm : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
		 pixelout : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		 ring : OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
		 sdaddr : OUT STD_LOGIC_VECTOR(11 DOWNTO 0);
		 zdataout : OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
	);
END COMPONENT;

COMPONENT cpcsymb
	PORT(sysclk : IN STD_LOGIC;
		 cpuclk : IN STD_LOGIC;
		 cpcrst : IN STD_LOGIC;
		 clkena_in : IN STD_LOGIC;
		 zwait : IN STD_LOGIC;
		 n_reset : IN STD_LOGIC;
		 crt_reset : IN STD_LOGIC;
		 kbc : IN STD_LOGIC;
		 kbd : IN STD_LOGIC;
		 crtclk : IN STD_LOGIC;
		 msci : IN STD_LOGIC;
		 msdi : IN STD_LOGIC;
		 enable : IN STD_LOGIC;
		 IO_in : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 Joya : IN STD_LOGIC_VECTOR(5 DOWNTO 0);
		 Joyb : IN STD_LOGIC_VECTOR(5 DOWNTO 0);
		 vdata : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 zdata : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 pixclk : OUT STD_LOGIC;
		 Hsync : OUT STD_LOGIC;
		 Vsync : OUT STD_LOGIC;
		 io_wr : OUT STD_LOGIC;
		 io_rd : OUT STD_LOGIC;
		 mem_wr : OUT STD_LOGIC;
		 mem_rd : OUT STD_LOGIC;
		 clkena : OUT STD_LOGIC;
		 cpcreset : OUT STD_LOGIC;
		 speed : OUT STD_LOGIC;
		 pal : OUT STD_LOGIC;
		 wsg : OUT STD_LOGIC;
		 menue : OUT STD_LOGIC;
		 data_write : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		 ldata : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		 rdata : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		 sdaddr : OUT STD_LOGIC_VECTOR(23 DOWNTO 0);
		 state : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
		 vaddr : OUT STD_LOGIC_VECTOR(23 DOWNTO 0);
		 Video : OUT STD_LOGIC_VECTOR(15 DOWNTO 0)
	);
END COMPONENT;

COMPONENT cfide
	PORT(cpuclk : IN STD_LOGIC;
		 n_reset : IN STD_LOGIC;
		 IO_wr : IN STD_LOGIC;
		 IO_rd : IN STD_LOGIC;
		 zclkena : IN STD_LOGIC;
		 rena_in : IN STD_LOGIC;
		 rIO_wr : IN STD_LOGIC;
		 rIO_rd : IN STD_LOGIC;
		 rmem_wr : IN STD_LOGIC;
		 rmem_rd : IN STD_LOGIC;
		 wsg : IN STD_LOGIC;
		 menue : IN STD_LOGIC;
		 addr : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		 cpudata_in : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 idedata_in : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		 raddr : IN STD_LOGIC_VECTOR(23 DOWNTO 0);
		 rcpudata_in : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 rmemdata : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 ide_css0 : OUT STD_LOGIC;
		 ide_csp0 : OUT STD_LOGIC;
		 ide_wr : OUT STD_LOGIC;
		 ide_rd : OUT STD_LOGIC;
		 nwait : OUT STD_LOGIC;
		 itsmy : OUT STD_LOGIC;
		 rmemwr : OUT STD_LOGIC;
		 rmem : OUT STD_LOGIC;
		 rena : OUT STD_LOGIC;
		 TxD : OUT STD_LOGIC;
		 rnwait : OUT STD_LOGIC;
		 ritsmy : OUT STD_LOGIC;
		 cpudata : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		 idea : OUT STD_LOGIC_VECTOR(2 DOWNTO 0);
		 idedata : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		 rcpudata : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		 sysctrl : OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
	);
END COMPONENT;

COMPONENT cyclone
	PORT(sysclk : IN STD_LOGIC;
		 IOB_clk : IN STD_LOGIC;
		 ide_css0 : IN STD_LOGIC;
		 ide_csp1 : IN STD_LOGIC;
		 ide_wr : IN STD_LOGIC;
		 ide_rd : IN STD_LOGIC;
		 ide_csp0 : IN STD_LOGIC;
		 TxD : IN STD_LOGIC;
		 kb_clki : IN STD_LOGIC;
		 kb_datai : IN STD_LOGIC;
		 ms_clki : IN STD_LOGIC;
		 ms_datai : IN STD_LOGIC;
		 target : IN STD_LOGIC;
		 addonin : IN STD_LOGIC_VECTOR(5 DOWNTO 0);
		 idea : IN STD_LOGIC_VECTOR(2 DOWNTO 0);
		 idedatai : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		 left : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		 right : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		 Ring : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
		 sleft : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		 sright : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		 ide1irq : OUT STD_LOGIC;
		 kb_clk : OUT STD_LOGIC;
		 kb_data : OUT STD_LOGIC;
		 ms_clk : OUT STD_LOGIC;
		 ms_data : OUT STD_LOGIC;
		 addon_out : OUT STD_LOGIC_VECTOR(8 DOWNTO 0);
		 idedatao : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		 JoyA : OUT STD_LOGIC_VECTOR(5 DOWNTO 0);
		 JoyB : OUT STD_LOGIC_VECTOR(5 DOWNTO 0)
	);
END COMPONENT;

COMPONENT clock
	PORT(inclk0 : IN STD_LOGIC;
		 c0 : OUT STD_LOGIC;
		 c1 : OUT STD_LOGIC;
		 c2 : OUT STD_LOGIC;
		 locked : OUT STD_LOGIC
	);
END COMPONENT;

COMPONENT shadowsymb
	PORT(sysclk : IN STD_LOGIC;
		 cpuclk : IN STD_LOGIC;
		 nreset : IN STD_LOGIC;
		 clkena : IN STD_LOGIC;
		 n_reset : IN STD_LOGIC;
		 kbc : IN STD_LOGIC;
		 kbd : IN STD_LOGIC;
		 crtclk : IN STD_LOGIC;
		 msci : IN STD_LOGIC;
		 msdi : IN STD_LOGIC;
		 enable : IN STD_LOGIC;
		 data_in : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 IO_in : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 vdata : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 pixclk : OUT STD_LOGIC;
		 Hsync : OUT STD_LOGIC;
		 Vsync : OUT STD_LOGIC;
		 io_wr : OUT STD_LOGIC;
		 io_rd : OUT STD_LOGIC;
		 mem_wr : OUT STD_LOGIC;
		 mem_rd : OUT STD_LOGIC;
		 shreset : OUT STD_LOGIC;
		 msc : OUT STD_LOGIC;
		 msd : OUT STD_LOGIC;
		 data_write : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		 ldata : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		 rdata : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		 sdaddr : OUT STD_LOGIC_VECTOR(23 DOWNTO 0);
		 state : OUT STD_LOGIC_VECTOR(1 DOWNTO 0);
		 vaddr : OUT STD_LOGIC_VECTOR(23 DOWNTO 0);
		 Video : OUT STD_LOGIC_VECTOR(15 DOWNTO 0)
	);
END COMPONENT;

COMPONENT scandoubler
	PORT(hsync_in : IN STD_LOGIC;
		 vsync_in : IN STD_LOGIC;
		 pixclk : IN STD_LOGIC;
		 bypass : IN STD_LOGIC;
		 video_in : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		 vsync_out : OUT STD_LOGIC;
		 hsync_out : OUT STD_LOGIC;
		 video_out : OUT STD_LOGIC_VECTOR(15 DOWNTO 0)
	);
END COMPONENT;

SIGNAL	cpcreset :  STD_LOGIC;
SIGNAL	cpuclk :  STD_LOGIC;
SIGNAL	crtclk :  STD_LOGIC;
SIGNAL	DI :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	DO :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	hsync :  STD_LOGIC;
SIGNAL	ide_css0 :  STD_LOGIC;
SIGNAL	ide_rd :  STD_LOGIC;
SIGNAL	ide_wr :  STD_LOGIC;
SIGNAL	idea :  STD_LOGIC_VECTOR(2 DOWNTO 0);
SIGNAL	idedata :  STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL	idedata_in :  STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL	IOB_clk :  STD_LOGIC;
SIGNAL	iord :  STD_LOGIC;
SIGNAL	iowr :  STD_LOGIC;
SIGNAL	itsmy :  STD_LOGIC;
SIGNAL	JoyA :  STD_LOGIC_VECTOR(5 DOWNTO 0);
SIGNAL	JoyB :  STD_LOGIC_VECTOR(5 DOWNTO 0);
SIGNAL	kbc :  STD_LOGIC;
SIGNAL	kbd :  STD_LOGIC;
SIGNAL	kcpvreset :  STD_LOGIC;
SIGNAL	ldata :  STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL	locked :  STD_LOGIC;
SIGNAL	ma :  STD_LOGIC_VECTOR(23 DOWNTO 0);
SIGNAL	mem_wr :  STD_LOGIC;
SIGNAL	menue :  STD_LOGIC;
SIGNAL	msc :  STD_LOGIC;
SIGNAL	msci :  STD_LOGIC;
SIGNAL	msd :  STD_LOGIC;
SIGNAL	msdi :  STD_LOGIC;
SIGNAL	nwait :  STD_LOGIC;
SIGNAL	pixclk :  STD_LOGIC;
SIGNAL	pixdata :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	raddr :  STD_LOGIC_VECTOR(23 DOWNTO 0);
SIGNAL	rcpudata :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	rIO_in :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	rdata :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	rena_in :  STD_LOGIC;
SIGNAL	resetin :  STD_LOGIC;
SIGNAL	rhsync :  STD_LOGIC;
SIGNAL	ring :  STD_LOGIC_VECTOR(3 DOWNTO 0);
SIGNAL	rIO_rd :  STD_LOGIC;
SIGNAL	rIO_wr :  STD_LOGIC;
SIGNAL	ritsmy :  STD_LOGIC;
SIGNAL	rmem :  STD_LOGIC;
SIGNAL	rmem_rd :  STD_LOGIC;
SIGNAL	rmem_wr :  STD_LOGIC;
SIGNAL	rnwait :  STD_LOGIC;
SIGNAL	rrena :  STD_LOGIC;
SIGNAL	rsate :  STD_LOGIC_VECTOR(1 DOWNTO 0);
SIGNAL	rsdata :  STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL	rvaddr :  STD_LOGIC_VECTOR(23 DOWNTO 0);
SIGNAL	rvideo :  STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL	rvsync :  STD_LOGIC;
SIGNAL	rwr :  STD_LOGIC;
SIGNAL	shreset :  STD_LOGIC;
SIGNAL	sleft :  STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL	speed :  STD_LOGIC;
SIGNAL	sright :  STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL	sysclk :  STD_LOGIC;
SIGNAL	sysctrl :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	TxD :  STD_LOGIC;
SIGNAL	vcaddr :  STD_LOGIC_VECTOR(23 DOWNTO 0);
SIGNAL	vena :  STD_LOGIC;
SIGNAL	video :  STD_LOGIC_VECTOR(15 DOWNTO 0);
SIGNAL	vrefresh :  STD_LOGIC;
SIGNAL	vsync :  STD_LOGIC;
SIGNAL	wsg :  STD_LOGIC;
SIGNAL	wsgin :  STD_LOGIC;
SIGNAL	zdata :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	zena_in :  STD_LOGIC;
SIGNAL	zena_o :  STD_LOGIC;
SIGNAL	Zreset :  STD_LOGIC;
SIGNAL	zstate :  STD_LOGIC_VECTOR(1 DOWNTO 0);
SIGNAL	zwait :  STD_LOGIC;


BEGIN 


b2v_inst : sdram
PORT MAP(zena_in => zena_in,
		 sysclk => sysclk,
		 reset => locked,
		 zwr => mem_wr,
		 wsgin => wsgin,
		 vrefresh => vrefresh,
		 rmem => rmem,
		 rwr => rwr,
		 rrefresh => '1',
		 speed => speed,
		 resetin => sysctrl(7) AND kcpvreset,
		 shresetin => sysctrl(6) OR shreset,
		 datawr => rcpudata,
		 mdb => mdb,
		 rAddr => raddr,
		 rstate => rsate,
		 sdata => DRAM_DQ,
		 vAddr => rvaddr OR vcaddr,
		 zAddr => ma,
		 zdatawr => DO,
		 zstate => zstate,
		 sd_we => DRAM_WE_N,
		 sd_ras => DRAM_RAS_N,
		 sd_cas => DRAM_CAS_N,
		 cpuclk => cpuclk,
		 rena_o => rrena,
		 zena_o => zena_o,
		 zwait => zwait,
		 reset_out => resetin,
		 c1MHz => crtclk,
		 IOB_clk => IOB_clk,
		 cpcreset => cpcreset,
		 shreset => Zreset,
		 wsg => wsg,
		 ba => DRAM_BA,
		 dataout => rdata,
		 dqm => DPM,
		 pixelout => pixdata,
		 ring => ring,
		 sdaddr => DRAM_ADDR,
		 zdataout => zdata);


b2v_inst1 : cpcsymb
PORT MAP(sysclk => sysclk,
		 cpuclk => cpuclk,
		 cpcrst => cpcreset,
		 clkena_in => (itsmy OR zena_o) AND nwait,
		 zwait => zwait,
		 n_reset => resetin,
		 crt_reset => Zreset,
		 kbc => kbc,
		 kbd => kbd,
		 crtclk => crtclk,
		 msci => msci,
		 msdi => msdi,
		 enable => sysctrl(6),
		 IO_in => DI,
		 Joya => JoyA,
		 Joyb => JoyB,
		 vdata => pixdata,
		 zdata => zdata,
		 Hsync => hsync,
		 Vsync => vsync,
		 io_wr => iowr,
		 io_rd => iord,
		 mem_wr => mem_wr,
		 clkena => zena_in,
		 cpcreset => kcpvreset,
		 speed => speed,
		 wsg => wsgin,
		 menue => menue,
		 data_write => DO,
		 ldata => ldata,
		 rdata => rsdata,
		 sdaddr => ma,
		 state => zstate,
		 vaddr => vcaddr,
		 Video => video);


rena_in <= (rrena OR ritsmy) AND rnwait;


b2v_inst14 : cfide
PORT MAP(cpuclk => cpuclk,
		 n_reset => Zreset,
		 IO_wr => iowr,
		 IO_rd => iord,
		 zclkena => zena_in,
		 rena_in => rena_in,
		 rIO_wr => rIO_wr,
		 rIO_rd => rIO_rd,
		 rmem_wr => rmem_wr,
		 rmem_rd => rmem_rd,
		 wsg => wsg,
		 menue => menue,
		 addr => ma(15 DOWNTO 0),
		 cpudata_in => DO,
		 idedata_in => idedata_in,
		 raddr => raddr,
		 rcpudata_in => rcpudata,
		 rmemdata => rdata,
		 ide_css0 => ide_css0,
		 ide_wr => ide_wr,
		 ide_rd => ide_rd,
		 nwait => nwait,
		 itsmy => itsmy,
		 rmemwr => rwr,
		 rmem => rmem,
		 TxD => TxD,
		 rnwait => rnwait,
		 ritsmy => ritsmy,
		 cpudata => DI,
		 idea => idea,
		 idedata => idedata,
		 rcpudata => rIO_in,
		 sysctrl => sysctrl);



b2v_inst21 : cyclone
PORT MAP(sysclk => sysclk,
		 IOB_clk => IOB_clk,
		 ide_css0 => ide_css0,
		 ide_csp1 => '1',
		 ide_wr => ide_wr,
		 ide_rd => ide_rd,
		 ide_csp0 => '1',
		 TxD => TxD,
		 kb_clki => '1',
		 kb_datai => '1',
		 ms_clki => msc,
		 ms_datai => msd,
		 target => cpcreset,
		 addonin => addbi,
		 idea => idea,
		 idedatai => idedata,
		 left => ldata,
		 right => rsdata,
		 Ring => ring,
		 sleft => sleft,
		 sright => sright,
		 kb_clk => kbc,
		 kb_data => kbd,
		 ms_clk => msci,
		 ms_data => msdi,
		 addon_out => addbo,
		 idedatao => idedata_in,
		 JoyA => JoyA,
		 JoyB => JoyB);


b2v_inst4 : clock
PORT MAP(inclk0 => clk_28,
		 c0 => sysclk,
		 c1 => DRAM_CLK,
		 locked => locked);


b2v_inst5 : shadowsymb
PORT MAP(sysclk => sysclk,
		 cpuclk => cpuclk,
		 nreset => Zreset,
		 clkena => rena_in,
		 n_reset => resetin,
		 kbc => kbc,
		 kbd => kbd,
		 crtclk => crtclk,
		 msci => msci,
		 msdi => msdi,
		 enable => NOT(sysctrl(6)),
		 data_in => rdata,
		 IO_in => rIO_in,
		 vdata => pixdata,
		 pixclk => pixclk,
		 Hsync => rhsync,
		 Vsync => rvsync,
		 io_wr => rIO_wr,
		 io_rd => rIO_rd,
		 mem_wr => rmem_wr,
		 mem_rd => rmem_rd,
		 shreset => shreset,
		 msc => msc,
		 msd => msd,
		 data_write => rcpudata,
		 ldata => sleft,
		 rdata => sright,
		 sdaddr => raddr,
		 state => rsate,
		 vaddr => rvaddr,
		 Video => rvideo);


b2v_inst7 : scandoubler
PORT MAP(hsync_in => rhsync OR hsync,
		 vsync_in => rvsync OR vsync,
		 pixclk => pixclk,
		 bypass => '0',
		 video_in => video OR rvideo,
		 vsync_out => nVSync,
		 hsync_out => nHSync,
		 video_out => dac);


vrefresh <= NOT(rvsync OR vsync OR rhsync OR hsync);

sigma1 <= '1';
sigma2 <= '1';
END bdf_type;