-- -----------------------------------------------------------------------
--
--                                 FPGA 64
--
--     A fully functional commodore 64 implementation in a single FPGA
--
-- -----------------------------------------------------------------------
-- Peter Wendrich (pwsoft@syntiac.com)
-- http://www.syntiac.com/fpga64.html
-- -----------------------------------------------------------------------
-- modify for CPC by TobiFlex
-- -----------------------------------------------------------------------

LIBRARY ieee;
USE ieee.std_logic_1164.all; 
--use IEEE.numeric_std.ALL;

ENTITY KBSCAN IS 
	PORT
	(
		EPclk :  IN  STD_LOGIC;
		nreset_in :  IN  STD_LOGIC;
		kb_clk :  IN  STD_LOGIC;
		kb_data :  IN  STD_LOGIC;
		sline :  IN  STD_LOGIC_VECTOR(3 DOWNTO 0);
		
		n_scread :  IN  STD_LOGIC;
		enable :  IN  STD_LOGIC;
		Joya	: IN STD_LOGIC_VECTOR(5 DOWNTO 0);
		Joyb	: IN STD_LOGIC_VECTOR(5 DOWNTO 0);		
		matrix :  OUT  STD_LOGIC_VECTOR(7 DOWNTO 0);
		nreset :  OUT  STD_LOGIC;
		speed :  OUT  STD_LOGIC;
		pal :  OUT  STD_LOGIC;
		wsg :  OUT  STD_LOGIC;
		
		scancode :  BUFFER  STD_LOGIC_VECTOR(7 DOWNTO 0);
		n_ready :  OUT  STD_LOGIC;
		extended :  OUT  STD_LOGIC;
		menue :  OUT  STD_LOGIC
	);
END KBSCAN;

ARCHITECTURE wire OF KBSCAN IS 

SIGNAL	releaseFlag :  STD_LOGIC;
SIGNAL	releaseFlagReg :  STD_LOGIC;
SIGNAL	extendedFlag :  STD_LOGIC;

	signal clk_reg: std_logic;
	signal clk_waitNextBit: std_logic;
	signal clk_filter: integer range 0 to 15;
	signal shift_reg:   STD_LOGIC_VECTOR(10 DOWNTO 0);

	signal bitsCount: integer range 0 to 10 := 0;
	signal timeout: integer range 0 to 5000 := 0; -- 2* 50 us at 50 Mhz


	signal key_zb0: std_logic;
	signal key_zb1: std_logic;
	signal key_zb2: std_logic;
	signal key_zb3: std_logic;
	signal key_zb4: std_logic;
	signal key_zb5: std_logic;
	signal key_zb6: std_logic;
	signal key_zb7: std_logic;
	signal key_zb8: std_logic;
	signal key_zb9: std_logic;
	signal key_zbdot: std_logic;
	signal key_zbenter: std_logic;
	signal key_caps: std_logic;
	signal key_clr: std_logic;
	signal key_lalt: std_logic;
	signal key_ralt: std_logic;
	signal key_tab: std_logic;

	signal key_del: std_logic;
	signal key_enter: std_logic;
	signal key_left: std_logic;
	signal key_right: std_logic;
	signal key_f12: std_logic;
	signal key_f7: std_logic;
	signal key_f1: std_logic;
	signal key_f3: std_logic;
	signal key_f5: std_logic;
	signal key_up: std_logic;
	signal key_down: std_logic;

	signal key_3: std_logic;
	signal key_W: std_logic;
	signal key_A: std_logic;
	signal key_4: std_logic;
	signal key_Z: std_logic;
	signal key_S: std_logic;
	signal key_E: std_logic;
	signal key_shiftl: std_logic;

	signal key_5: std_logic;
	signal key_R: std_logic;
	signal key_D: std_logic;
	signal key_6: std_logic;
	signal key_C: std_logic;
	signal key_F: std_logic;
	signal key_T: std_logic;
	signal key_X: std_logic;
	
	signal key_7: std_logic;
	signal key_Y: std_logic;
	signal key_G: std_logic;
	signal key_8: std_logic;
	signal key_B: std_logic;
	signal key_H: std_logic;
	signal key_U: std_logic;
	signal key_V: std_logic;

	signal key_9: std_logic;
	signal key_I: std_logic;
	signal key_J: std_logic;
	signal key_0: std_logic;
	signal key_M: std_logic;
	signal key_K: std_logic;
	signal key_O: std_logic;
	signal key_N: std_logic;

	signal key_P: std_logic;
	signal key_L: std_logic;
	signal key_minus: std_logic;
	signal key_dot: std_logic;
	signal key_colon: std_logic;
	signal key_at: std_logic;
	signal key_comma: std_logic;

	signal key_star: std_logic;
	signal key_semicolon: std_logic;
	signal key_shiftr: std_logic;
	signal key_equal: std_logic;
	signal key_arrowup: std_logic;
	signal key_slash: std_logic;

	signal key_1: std_logic;
	signal key_arrowleft: std_logic;
	signal key_ctrl: std_logic;
	signal key_ctrlr: std_logic;
	signal key_2: std_logic;
	signal key_space: std_logic;
	signal key_Q: std_logic;
	signal key_esc: std_logic;

BEGIN 

--	nreset <= '0' WHEN nreset_in='0' OR (key_F12='1' AND (key_shiftl OR key_shiftr)) ELSE '1';
	nreset <= '0' WHEN nreset_in='0' OR (scancode=X"07" AND (key_shiftl OR key_shiftr)='1') ELSE '1';
	menue <= '1' WHEN (scancode=X"07" AND (key_shiftl OR key_shiftr)='0') ELSE '0';
	PROCESS(nreset_in, EPclk)
	BEGIN
		IF RISING_EDGE(EPclk) THEN
		matrix <="11111111";
		IF enable='1' THEN
		CASE sline IS 
			WHEN "0000" => 	matrix(0) <= NOT key_up;
							matrix(1) <= NOT key_right;
							matrix(2) <= NOT key_down;
							matrix(3) <= NOT key_zb9;
							matrix(4) <= NOT key_zb6;
							matrix(5) <= NOT key_zb3;
							matrix(6) <= NOT key_zbenter;
							matrix(7) <= NOT key_zbdot;
							
			WHEN "0001" => 	matrix(0) <= NOT key_left;
							matrix(1) <= NOT (key_lalt OR key_ralt);	--copy
							matrix(2) <= NOT key_zb7;
							matrix(3) <= NOT key_zb8;
							matrix(4) <= NOT key_zb5;
							matrix(5) <= NOT key_zb1;
							matrix(6) <= NOT key_zb2;
							matrix(7) <= NOT key_zb0;
							
			WHEN "0010" => 	matrix(0) <= NOT key_clr;
							matrix(1) <= NOT key_at;
							matrix(2) <= NOT key_enter;
							matrix(3) <= NOT key_star;
							matrix(4) <= NOT key_zb4;
							matrix(5) <= NOT (key_shiftl OR key_shiftr);
							matrix(6) <= NOT key_arrowleft;
							matrix(7) <= NOT (key_ctrl OR key_ctrlr);
							
			WHEN "0011" => 	matrix(0) <= NOT key_equal;
							matrix(1) <= NOT key_minus;
							matrix(2) <= NOT key_arrowup;
							matrix(3) <= NOT key_P;
							matrix(4) <= NOT key_colon;
							matrix(5) <= NOT key_semicolon;
							matrix(6) <= NOT key_slash;
							matrix(7) <= NOT key_dot;
							
			WHEN "0100" => 	matrix(0) <= NOT key_0;
							matrix(1) <= NOT key_9;
							matrix(2) <= NOT key_O;
							matrix(3) <= NOT key_I;
							matrix(4) <= NOT key_L;
							matrix(5) <= NOT key_K;
							matrix(6) <= NOT key_M;
							matrix(7) <= NOT key_comma;
							
			WHEN "0101" => 	matrix(0) <= NOT key_8;
							matrix(1) <= NOT key_7;
							matrix(2) <= NOT key_U;
							matrix(3) <= NOT key_Y;
							matrix(4) <= NOT key_H;
							matrix(5) <= NOT key_J;
							matrix(6) <= NOT key_N;
							matrix(7) <= NOT key_space;
							
			WHEN "0110" => 	matrix(0) <= NOT key_6 AND Joya(0);
							matrix(1) <= NOT key_5 AND Joya(1);
							matrix(2) <= NOT key_R AND Joya(2);
							matrix(3) <= NOT key_T AND Joya(3);
							matrix(4) <= NOT key_G AND Joya(4);
							matrix(5) <= NOT key_F AND Joya(5);
							matrix(6) <= NOT key_B;
							matrix(7) <= NOT key_V;
							
			WHEN "0111" => 	matrix(0) <= NOT key_4;
							matrix(1) <= NOT key_3;
							matrix(2) <= NOT key_E;
							matrix(3) <= NOT key_W;
							matrix(4) <= NOT key_S;
							matrix(5) <= NOT key_D;
							matrix(6) <= NOT key_C;
							matrix(7) <= NOT key_X;
			WHEN "1000" => 	matrix(0) <= NOT key_1;
			
							matrix(1) <= NOT key_2;
							matrix(2) <= NOT key_esc;
							matrix(3) <= NOT key_Q;
							matrix(4) <= NOT key_tab;
							matrix(5) <= NOT key_A;
							matrix(6) <= NOT key_caps;
							matrix(7) <= NOT key_Z;
			WHEN "1001" => 	--matrix(0) <= NOT key_up;
							--matrix(1) <= NOT key_right;
							--matrix(2) <= NOT key_down;
							--matrix(3) <= NOT key_9;
							--matrix(4) <= NOT key_6;
							--matrix(5) <= NOT key_3;
							--matrix(6) <= NOT key_return;
							matrix(0) <= Joyb(0);
							matrix(1) <= Joyb(1);
							matrix(2) <= Joyb(2);
							matrix(3) <= Joyb(3);
							matrix(4) <= Joyb(4);
							matrix(5) <= Joyb(5);
							
							matrix(7) <= NOT key_del;
			WHEN OTHERS => 	null;
		END CASE;
		END IF;
		END IF;
	
		releaseFlag <= releaseFlagReg OR NOT enable;	
		
		IF  nreset_in='0' THEN
			extendedFlag <= '0';
			releaseFlagReg <= '0';
			wsg <= '0';
			speed <= '0';
			pal <= '1';
			bitsCount <= 0;
			n_ready <= '1';
			
		key_zb0 <= '0';
		key_zb1 <= '0';
		key_zb2 <= '0';
		key_zb3 <= '0';
		key_zb4 <= '0';
		key_zb5 <= '0';
		key_zb6 <= '0';
		key_zb7 <= '0';
		key_zb8 <= '0';
		key_zb9 <= '0';
		key_zbdot <= '0';
		key_zbenter <= '0';
		key_caps <= '0';
		key_clr <= '0';
		key_lalt <= '0';
		key_ralt <= '0';
		key_tab <= '0';

		key_F12 <= '0';
		key_del <= '0';
		key_enter <= '0';
		key_left <= '0';
		key_right <= '0';
		key_f7 <= '0';
		key_f1 <= '0';
		key_f3 <= '0';
		key_f5 <= '0';
		key_up <= '0';
		key_down <= '0';

		key_3 <= '0';
		key_W <= '0';
		key_A <= '0';
		key_4 <= '0';
		key_Z <= '0';
		key_S <= '0';
		key_E <= '0';
		key_shiftl <= '0';

		key_5 <= '0';
		key_R <= '0';
		key_D <= '0';
		key_6 <= '0';
		key_C <= '0';
		key_F <= '0';
		key_T <= '0';
		key_X <= '0';
	
		key_7 <= '0';
		key_Y <= '0';
		key_G <= '0';
		key_8 <= '0';
		key_B <= '0';
		key_H <= '0';
		key_U <= '0';
		key_V <= '0';

		key_9 <= '0';
		key_I <= '0';
		key_J <= '0';
		key_0 <= '0';
		key_M <= '0';
		key_K <= '0';
		key_O <= '0';
		key_N <= '0';

		key_P <= '0';
		key_L <= '0';
		key_minus <= '0';
		key_dot <= '0';
		key_colon <= '0';
		key_at <= '0';
		key_comma <= '0';

		key_star <= '0';
		key_semicolon <= '0';
		key_shiftr <= '0';
		key_equal <= '0';
		key_arrowup <= '0';
		key_slash <= '0';

		key_1 <= '0';
		key_arrowleft <= '0';
		key_ctrl <= '0';
		key_ctrlr <= '0';
		key_2 <= '0';
		key_space <= '0';
		key_Q <= '0';
		key_esc <= '0';
			
			
		ELSIF RISING_EDGE(EPclk) THEN

			-- Timeout if keyboard does not send anymore.
			if timeout /= 0 then
				timeout <= timeout - 1;
			else
				bitsCount <= 0;
			end if;
			
			if n_scread='0' then
				n_ready <= '1';
			end if;
			
			-- Filter glitches on the clock
			if (clk_reg /= kb_clk) then
--				clk_filter <= 15; -- Wait 15 ticks 
				clk_filter <= 5; -- Wait 15 ticks 
				clk_reg <= kb_clk; -- Store clock edge to detect changes
				clk_waitNextBit <= '0'; -- Next bit comming up...
			elsif (clk_filter /= 0) then
				-- Wait for clock to stabilise
				-- Clock must be stable before we sample the data line.
				clk_filter <= clk_filter - 1;
			elsif (clk_reg = '1') and (clk_waitNextBit = '0') then
				-- We have a stable clock, so assume stable data too.
				clk_waitNextBit <= '1';
				
				-- Move data into shift register
				shift_reg <= kb_data & shift_reg(10 downto 1);
				timeout <= 5000;
				if bitsCount < 10 then
					bitsCount <= bitsCount + 1;
				else
					-- 10 bits received. Output new scancode
					bitsCount <= 0;
					IF shift_reg(9 downto 2)=X"E0" THEN		--Extended
						extendedFlag <= '1';
					ELSIF shift_reg(9 downto 2)=X"F0" THEN	--BREAK
						releaseFlagReg <= '1';
					ELSE									--make
						scancode <= (releaseFlag&shift_reg(8 downto 2));
						extended <= extendedFlag;
						n_ready <= '0';


						extendedFlag <= '0';
						releaseFlagReg <= '0';
						
						case shift_reg(9 downto 2) is
							when X"03" => pal <= '1';		--F5 50Hz
							when X"0B" => pal <= '0';		--F6 60Hz
							when X"01" => 					--F9 Clone
											wsg <= '0';
											speed <= '0';
							when X"09" => 					--F10 Turbo
											wsg <= '0';
											speed <= '1';
							when X"78" => 					--F11 Wahnsinnige Geschwindigkeit
											wsg <= '1';
											speed <= '1';
							when X"07" => key_F12 <= not releaseFlag;
--								if releaseFlag = '0' then
--									ntsc <= NOT ntsc
--								end if;
--							when X"03" => key_F5 <= not releaseFlag;
--							when X"04" => key_F3 <= not releaseFlag;
--							when X"05" => key_F1 <= not releaseFlag;
--							when X"06" => -- F2
--								if releaseFlag = '0' then
--		--							traceKey <= '1';
--								end if;
--							when X"83" => key_F7 <= not releaseFlag;
							
							when X"0D" => key_tab <= not releaseFlag;
							when X"0E" => key_arrowleft <= not releaseFlag;
--							when X"11" => key_copy <= not releaseFlag; 
							when X"11" => if extendedFlag = '0' then key_lalt <= not releaseFlag; else key_ralt <= not releaseFlag; end if;
							when X"12" => if extendedFlag = '0' then key_shiftl <= not releaseFlag; end if;
							when X"14" => if extendedFlag = '0' then key_ctrl <= not releaseFlag; else key_ctrlr <= not releaseFlag; end if;
							when X"15" => key_Q <= not releaseFlag; 
							when X"16" => key_1 <= not releaseFlag; 
							when X"1A" => key_Z <= not releaseFlag; 
							when X"1B" => key_S <= not releaseFlag; 
							when X"1C" => key_A <= not releaseFlag; 
							when X"1D" => key_W <= not releaseFlag; 
							when X"1E" => key_2 <= not releaseFlag; 
							when X"21" => key_C <= not releaseFlag; 
							when X"22" => key_X <= not releaseFlag; 
							when X"23" => key_D <= not releaseFlag; 
							when X"24" => key_E <= not releaseFlag; 
							when X"25" => key_4 <= not releaseFlag; 
							when X"26" => key_3 <= not releaseFlag; 
							when X"29" => key_space <= not releaseFlag; 
							when X"2A" => key_V <= not releaseFlag; 
							when X"2B" => key_F <= not releaseFlag; 
							when X"2C" => key_T <= not releaseFlag; 
							when X"2D" => key_R <= not releaseFlag; 
							when X"2E" => key_5 <= not releaseFlag; 
							when X"31" => key_N <= not releaseFlag; 
							when X"32" => key_B <= not releaseFlag; 
							when X"33" => key_H <= not releaseFlag; 
							when X"34" => key_G <= not releaseFlag; 
							when X"35" => key_Y <= not releaseFlag; 
							when X"36" => key_6 <= not releaseFlag; 
							when X"3A" => key_M <= not releaseFlag; 
							when X"3B" => key_J <= not releaseFlag; 
							when X"3C" => key_U <= not releaseFlag; 
							when X"3D" => key_7 <= not releaseFlag; 
							when X"3E" => key_8 <= not releaseFlag;
							when X"41" => key_comma <= not releaseFlag; 
							when X"42" => key_K <= not releaseFlag;
							when X"43" => key_I <= not releaseFlag; 
							when X"44" => key_O <= not releaseFlag; 
							when X"45" => key_0 <= not releaseFlag; 
							when X"46" => key_9 <= not releaseFlag; 
							when X"49" => key_dot <= not releaseFlag; 
							when X"4A" => if extendedFlag = '0' then key_slash <= not releaseFlag; end if; 
							when X"4B" => key_L <= not releaseFlag; 
							when X"4C" => key_colon <= not releaseFlag; 
							when X"4D" => key_P <= not releaseFlag; 
							when X"4E" => key_minus <= not releaseFlag;
							when X"52" => key_semicolon <= not releaseFlag; 
							when X"54" => key_at <= not releaseFlag; 
							when X"55" => key_equal <= not releaseFlag;
							when X"58" => key_caps <= not releaseFlag;
							when X"59" => key_shiftr <= not releaseFlag;
							when X"5A" => if extendedFlag = '0' then key_enter <= not releaseFlag; else key_zbenter <= not releaseFlag; end if;
							when X"5B" => key_star <= not releaseFlag; 
							when X"5D" => key_arrowup <= not releaseFlag;
							when X"66" => key_del <= not releaseFlag; 
							when X"69" => if extendedFlag = '0' then key_zb1 <= not releaseFlag; end if; 
							when X"6B" => if extendedFlag = '0' then key_zb4 <= not releaseFlag; else key_left <= not releaseFlag; end if;
							when X"6C" => if extendedFlag = '0' then key_zb7 <= not releaseFlag; end if; 
							
							when X"70" => if extendedFlag = '0' then key_zb0 <= not releaseFlag; end if; 
							when X"71" => if extendedFlag = '0' then key_zbdot <= not releaseFlag; else key_clr <= not releaseFlag; end if;
							when X"72" => if extendedFlag = '0' then key_zb2 <= not releaseFlag; else key_down <= not releaseFlag; end if;
							when X"73" => key_zb5 <= not releaseFlag; 
							when X"74" => if extendedFlag = '0' then key_zb6 <= not releaseFlag; else key_right <= not releaseFlag; end if;
							when X"75" => if extendedFlag = '0' then key_zb8 <= not releaseFlag; else key_up <= not releaseFlag; end if;
							when X"76" => key_esc <= not releaseFlag; 
							when X"7A" => if extendedFlag = '0' then key_zb3 <= not releaseFlag; end if; 
							when X"7D" => if extendedFlag = '0' then key_zb9 <= not releaseFlag; end if; 
							when others => null;
						end case;
					END IF;
				END IF;
			END IF;
		END IF;
	END PROCESS;
END;