------------------------------------------------------------------------------
------------------------------------------------------------------------------
--                                                                          --
-- Copyright (c) 2006-2009 Tobias Gubener                                   -- 
-- Subdesign CPC T-REX by TobiFlex                                          --
--                                                                          --
-- This source file is free software: you can redistribute it and/or modify --
-- it under the terms of the GNU General Public License as published        --
-- by the Free Software Foundation, either version 3 of the License, or     --
-- (at your option) any later version.                                      --
--                                                                          --
-- This source file is distributed in the hope that it will be useful,      --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of           --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            --
-- GNU General Public License for more details.                             --
--                                                                          --
-- You should have received a copy of the GNU General Public License        --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.    --
--                                                                          --
------------------------------------------------------------------------------
------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity Z80_timing is
   port(        
		data_in			  : in std_logic_vector(7 downto 0);
		clk               : in std_logic;
		reset               : in std_logic;
		int_in               : in std_logic;
        clkena_in         : in std_logic;
        n_wait           : in std_logic;
		i_state		      : in std_logic_vector(10 downto 0);--extra_one&ixye&fetchOPC&decodeOPC&dirio&state
        wsg           : in std_logic;
		data_out			  : out std_logic_vector(7 downto 0);
		o_extra_cycle	  : out std_logic_vector(2 downto 0);
        int_out            : out std_logic;
        clkena            : out std_logic;
        n_wait_out           : out std_logic
        );
end Z80_timing;

ARCHITECTURE logic OF Z80_timing IS

   SIGNAL n_waitd    : std_logic;
   SIGNAL expect_wait	  : std_logic;
   SIGNAL test_wait	  : std_logic;
   SIGNAL io_wait	  : std_logic;
   SIGNAL decodeOPC	  : std_logic;
   SIGNAL fetchOPC	  : std_logic;
   SIGNAL extra_cycle : std_logic_vector(2 downto 0);
   SIGNAL state : std_logic_vector(1 downto 0);
   SIGNAL wait_three    : std_logic;
   SIGNAL wait_four    : std_logic;
   SIGNAL wait_five	  : std_logic;
   SIGNAL wait_seven  : std_logic;
   SIGNAL int_ack    : std_logic;
   SIGNAL special_cycle    : std_logic;
   SIGNAL wait_cnt : std_logic_vector(1 downto 0);
   SIGNAL data_ind : std_logic_vector(7 downto 0);




BEGIN  
	state <= i_state(1 downto 0);
	decodeOPC <= i_state(3);
	fetchOPC <= i_state(4);
	int_ack <= i_state(5);
	wait_three <= i_state(6);
	wait_four <= i_state(7);
	wait_five <= i_state(8);
	wait_seven <= i_state(9);

	o_extra_cycle <= extra_cycle;
	
	n_wait_out <= n_wait;--'1' WHEN wait_cnt="00" ELSE '0';
	
	special_cycle <= '1' WHEN state="01" AND (decodeOPC OR wait_three OR wait_four OR wait_five OR wait_seven)='0' ELSE '0';
	expect_wait <= '1' WHEN extra_cycle="001" AND (fetchOPC='1' AND n_wait='0') ELSE '0';

	clkena <= '1' WHEN clkena_in='1' AND ((extra_cycle="001" AND expect_wait='0') OR special_cycle='1' OR wsg='1')
					ELSE '0';
PROCESS (reset, clk, n_wait, wsg, clkena_in, data_in, data_ind)
	BEGIN
		IF wsg='1' THEN
			data_out <= data_in;
		ELSE	
			data_out <= data_ind;
		END IF;
		IF reset='0' THEN
			extra_cycle <= "000";
		ELSIF rising_edge(clk) THEN
        	IF clkena_in='1' THEN
				IF n_wait='1' THEN
					data_ind <= data_in;
				END IF;
				n_waitd <= n_wait;
				int_out <= int_in;
				wait_cnt <= wait_cnt+1;
				test_wait <= '0';
				io_wait <= '0';
				IF extra_cycle="000" THEN
					IF i_state(2)='1' AND state(1)='1' THEN	--IORQ
						io_wait <= '1';
						extra_cycle <= "011";
					ELSIF wait_three='1' THEN
						extra_cycle <= "010";
					ELSIF wait_four='1' OR ((int_ack AND fetchOPC)='1') THEN
						extra_cycle <= "011";
					ELSIF wait_five='1' THEN
						extra_cycle <= "100";
					ELSIF wait_seven='1' THEN
						extra_cycle <= "110";
					ELSIF decodeOPC='1' OR fetchOPC='1' THEN
						extra_cycle <= "001";
					ELSIF special_cycle='1' THEN
						extra_cycle <= "000";
					ELSE	
						extra_cycle <= "010";
					END IF;
				ELSE	
					IF (test_wait='0' OR n_wait='1') AND expect_wait='0' THEN
						extra_cycle <= extra_cycle-1;
					END IF;
				END IF;	
				IF (extra_cycle="000" AND (i_state(4 downto 1)="0001" OR i_state(4 downto 0)="00000")) OR io_wait='1' OR (test_wait='1' AND n_wait='0')  THEN
					test_wait <= '1';
				END IF;
				
			END IF;
		END IF;	
	END PROCESS;
END;	