------------------------------------------------------------------------------
------------------------------------------------------------------------------
--                                                                          --
-- Copyright (c) 2008-2009 Tobias Gubener                                   -- 
-- Subdesign CPC T-REX by TobiFlex                                          --
--                                                                          --
-- This source file is free software: you can redistribute it and/or modify --
-- it under the terms of the GNU General Public License as published        --
-- by the Free Software Foundation, either version 3 of the License, or     --
-- (at your option) any later version.                                      --
--                                                                          --
-- This source file is distributed in the hope that it will be useful,      --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of           --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            --
-- GNU General Public License for more details.                             --
--                                                                          --
-- You should have received a copy of the GNU General Public License        --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.    --
--                                                                          --
------------------------------------------------------------------------------
------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

 
entity cfide is
   port ( 
	cpuclk: in std_logic;	
	n_reset: in std_logic;	
	addr: in std_logic_vector(15 downto 0);		
	cpudata_in: in std_logic_vector(7 downto 0);
	IO_wr: in std_logic;			
	IO_rd: in std_logic;			
	zclkena: in std_logic;			
	
	idedata: out std_logic_vector(15 downto 0);		
	idea: buffer std_logic_vector(2 downto 0);		
	ide_css0: out std_logic;			
	ide_csp0: out std_logic;			
	ide_wr: out std_logic;			
	ide_rd: out std_logic;			
			
	cpudata: out std_logic_vector(7 downto 0);		
	nwait: buffer std_logic;			
	itsmy: out std_logic;
	
	rena_in: in std_logic;			
	raddr: in std_logic_vector(23 downto 0);		
	rcpudata_in: in std_logic_vector(7 downto 0);
	rmemdata: in std_logic_vector(7 downto 0);
	rIO_wr: in std_logic;			
	rIO_rd: in std_logic;			
	rmem_wr: in std_logic;			
	rmem_rd: in std_logic;			
	wsg: in std_logic;			
	rcpudata: out std_logic_vector(7 downto 0);	
	rmemwr: out std_logic;	
	rmem: buffer std_logic;	
	rena: out std_logic;	
	TxD: out std_logic;	

	idedata_in: in std_logic_vector(15 downto 0);	
	
	rnwait: buffer std_logic;			
	ritsmy: out std_logic;
	sysctrl: out std_logic_vector(7 downto 0);	
	menue: in std_logic			


		
   );

end cfide;


architecture wire of cfide is

signal ide_select: std_logic;
signal fdc_select: std_logic;
signal IDErd_data: std_logic_vector(15 downto 0);
signal IDEwr_data: std_logic_vector(7 downto 0);
signal IDEwait: std_logic_vector(6 downto 0);
signal IOcpuena: std_logic;
signal start_now: std_logic;
signal highbyte: std_logic;		--0= bit 7..0   1= bit 15..8

type micro_states is (idle, ide1, ide6, ide7);
signal micro_state		: micro_states;
signal next_micro_state		: micro_states;


signal FDCstat: std_logic_vector(7 downto 0);
signal FDCdata: std_logic_vector(7 downto 0);
signal FDCdir: std_logic;

signal shift: std_logic_vector(9 downto 0);
signal clkgen: std_logic_vector(9 downto 0);
signal shiftout: std_logic;
signal txbusy: std_logic;
signal ld: std_logic;
signal rs232_select: std_logic;


signal ride_select: std_logic;
signal rstart_now: std_logic;
signal selide: std_logic;
signal rIOcpuena: std_logic;

--RISC FDC EMU
type FDC_states is (fdcnew, command, execute, result);
signal FDC_state		: FDC_states;

signal commandcount: std_logic_vector(3 downto 0);
signal resultcount: std_logic_vector(2 downto 0);




begin

itsmy <= ide_select;	

cpudata <=  FDCdata WHEN fdc_select='1' AND addr(0)='1' ELSE
			(FDCstat(7 downto 4)&"0000") WHEN fdc_select='1' AND addr(0)='0' ELSE
			"00000000" WHEN ide_select='0' ELSE 
			IDErd_data(7 downto 0) WHEN highbyte='0' ELSE 
			IDErd_data(15 downto 8);
			
nwait <= '1' WHEN ide_select='0' OR IOcpuena='1' ELSE '0'; 

ide_select <= '1' when (IO_wr AND IO_RD)='0' AND addr(15 downto 4)=X"FD0" ELSE '0';--Primr IDE
fdc_select <= '1' when (IO_wr AND IO_RD)='0' AND (addr(15 downto 0)=X"FB7E" OR addr(15 downto 0)=X"FB7F") ELSE '0';--FB7E/F
--fdc_select <= '1' when (IO_wr AND IO_RD)='0' AND (addr(15 downto 0)=X"FBF6" OR addr(15 downto 0)=X"FBF7") ELSE '0';--FBF6/7 Vortex

ride_select <= '1' when (rIO_wr AND rIO_RD)='0' AND raddr(15 downto 4)=X"FD0" ELSE '0';--Primr IDE
ritsmy <= ride_select;	
rnwait <= '1' WHEN ride_select='0' OR rIOcpuena='1' ELSE '0'; 

-----------------------------------------------------------------
-- FDC CPC Side
-----------------------------------------------------------------
process(cpuclk, FDCdata, FDCdir, FDCstat, FDC_state, n_reset, rIO_rd, raddr, txbusy, rmemdata, IDErd_data, highbyte, menue, ride_select)
begin
	IF n_reset='0' THEN
		FDCdir <= '0';
		FDCstat <= "00000000";
   	ELSIF cpuclk'event AND cpuclk = '1' THEN


		IF fdc_select='1' AND IO_wr='0' AND zclkena='1' THEN
			FDCdata <= cpudata_in;
		ELSIF raddr(15 downto 0)=X"FC05" AND rena_in='1' THEN 
			IF rIO_wr='0' THEN	--OUT FC05
				FDCdata <= rcpudata_in;
			END IF;		
		END IF;		
	
		IF fdc_select='1' AND (IO_wr='0' OR (IO_rd='0' AND addr(0)='1')) AND zclkena='1' THEN
			FDCstat(7) <= '0';
			FDCstat(6 downto 4) <= FDCstat(2 downto 0);	
			FDCdir <= IO_wr; 
		END IF;
--	end if;
--end process; 
--
-------------------------------------------------------------------
----FDC RISC Side
-------------------------------------------------------------------			
--process(cpuclk)
--begin
--	IF cpuclk'event AND cpuclk = '1' THEN
		IF raddr(15 downto 0)=X"FC04" THEN 
			IF rIO_wr='0' AND rena_in='1' THEN	--OUT FC04
				IF rcpudata_in(7)='1' THEN
					FDCstat(6 downto 4) <= rcpudata_in(6 downto 4);
				END IF;	
				FDCstat(7) <= '1';
				FDCstat(3 downto 0) <= '0'&rcpudata_in(2 downto 0);
			END IF;	
		END IF;	
	END IF;		
		
-------------------------------------------------------------------
----FDC RISC EMU
-------------------------------------------------------------------			
	IF n_reset='0' THEN
		FDC_state <= fdcnew;
		commandcount <= "0000";
		resultcount <= "000";
		sysctrl <= "00000000";
	ELSIF cpuclk'event AND cpuclk = '1' THEN
		IF rena_in='1' THEN 
			IF raddr(15 downto 0)=X"FC08" AND rIO_wr='0' THEN 	--OUT FC08
				IF rcpudata_in(7)='1' THEN
					sysctrl(7) <= rcpudata_in(0);		--reset
				END IF;
				IF rcpudata_in(6)='1' THEN
					sysctrl(6) <= rcpudata_in(0);		--riscview
				END IF;
				IF rcpudata_in(5)='1' THEN
					sysctrl(5) <= rcpudata_in(0);
				END IF;
				IF rcpudata_in(4)='1' THEN
					sysctrl(4) <= rcpudata_in(0);
				END IF;
				IF rcpudata_in(3)='1' THEN
					sysctrl(3) <= rcpudata_in(0);
				END IF;
				IF rcpudata_in(2)='1' THEN
					sysctrl(2) <= rcpudata_in(0);
				END IF;
				IF rcpudata_in(1)='1' THEN
					sysctrl(1) <= rcpudata_in(0);
				END IF;
			END IF;	
				
			IF(raddr(15 downto 0)=X"FB7E" OR raddr(15 downto 0)=X"FB7F") THEN	--FB7E STATUS  FB7F DATA
				CASE FDC_state IS
					WHEN fdcnew	=>
							IF rIO_wr='0' AND raddr(0)='1' THEN
								IF rcpudata_in=X"03" THEN		--SPECIFY
									commandcount <= "0010";
									FDC_state <= command;
								ELSIF rcpudata_in=X"04" THEN	--SENSE DRIVE STATUS
									commandcount <= "0001";
									resultcount <= "001";
									FDC_state <= command;
								ELSIF rcpudata_in=X"07" THEN		--RECALIBRATE
									commandcount <= "0001";
									FDC_state <= command;
								ELSIF rcpudata_in=X"08" THEN	--SENSE INTERRUPT STATUS
									resultcount <= "010";
									FDC_state <= result;
								END IF;
							END IF;
					WHEN command=>	
							IF rIO_wr='0' AND raddr(0)='1' THEN
								IF commandcount="0001" THEN
									IF resultcount="000" THEN
										FDC_state <= fdcnew;
									ELSE
										FDC_state <= result;
									END IF;	
								END IF;
								commandcount <= commandcount-1;
							END IF;
					WHEN result	=>
							IF rIO_rd='0' AND raddr(0)='1' THEN
								IF resultcount="001" THEN
									FDC_state <= fdcnew;
								END IF;
								resultcount <= resultcount-1;
							END IF;
					WHEN OTHERS => null;
				END CASE;
			END IF;	
		END IF;
	END IF;
	
	
	
	rcpudata <= "00000000";
	IF rIO_rd='0' THEN
		IF raddr(15 downto 0)=X"FC04" THEN	--IN FC04
			rcpudata(7 downto 6) <= FDCstat(7)&FDCdir;	--FDCstat lesen
			rcpudata(1) <= menue;
			rcpudata(0) <= txbusy;
		END IF;	
		IF raddr(15 downto 0)=X"FC05" THEN	--IN FC05
			rcpudata <= FDCdata;
		END IF;	
		IF (raddr(15 downto 0)=X"FB7E" OR raddr(15 downto 0)=X"FB7F") THEN	--FB7E STATUS  FB7F DATA
			IF raddr(0)='0' THEN	--status
				rcpudata <= "10000000";--&txbusy;
				IF FDC_state=result THEN
					rcpudata(6) <= '1';
				END IF;
				IF FDC_state/=fdcnew THEN
					rcpudata(4) <= '1';
				END IF;
			ELSE					--result
				rcpudata <= "00100000";		--ready by SENSE DRIVE STATUS
			END IF;
		END IF;	
		IF ride_select='1' THEN	
			IF highbyte='0' THEN
				rcpudata <= IDErd_data(7 downto 0);
			ELSE	 
				rcpudata <= IDErd_data(15 downto 8);
			END IF;
		END IF;
	END IF;
	
end process; 

rmem <= '0' when (rmem_wr AND rmem_rd)='0' ELSE '1';
rmemwr <= '0' when rmem_wr='0' ELSE '1';
rena <= '0' WHEN (rmem='0' AND rena_in='0') OR (RS232_select='1' AND txbusy='1' AND rIO_wr='0') ELSE '1';
RS232_select <= '1' when (rIO_wr AND rIO_RD)='0' AND raddr(15 downto 0)=X"FC07" ELSE '0';--FC07

-----------------------------------------------------------------
-- IDE States
-----------------------------------------------------------------
start_now <= '1' WHEN addr(2 downto 0)/="000" OR (IO_wr='0' AND highbyte='1') OR (IO_rd='0' AND highbyte='0') ELSE '0';
rstart_now <= '1' WHEN raddr(2 downto 0)/="000" OR (rIO_wr='0' AND highbyte='1') OR (rIO_rd='0' AND highbyte='0') ELSE '0';

idedata <= rcpudata_in&IDEwr_data WHEN selide='0' ELSE cpudata_in&IDEwr_data;
idea <= raddr(2 downto 0) WHEN selide='0' ELSE addr(2 downto 0);

process(n_reset, cpuclk)
begin
	IF n_reset='0' THEN
		highbyte <= '0';
		micro_state <= idle;
		IOcpuena <= '0';
		rIOcpuena <= '0';
		ide_wr <= '1';
		ide_rd <= '1';
		ide_csp0 <= '1';
		ide_css0 <= '1';
   	ELSIF cpuclk'event AND cpuclk = '1' THEN
		micro_state <= idle;
		IOcpuena <= '0';
		rIOcpuena <= '0';
		CASE micro_state IS
			WHEN idle => 
				ide_wr <= '1';
				ide_rd <= '1';
				ide_csp0 <= '1';
				ide_css0 <= '1';
				selide <= '0';
				IF ide_select='1' THEN
					selide <= '1';
					IF addr(2 downto 0)/="000" OR highbyte='0' THEN
						highbyte <= '0';
						IDEwr_data <= cpudata_in;
					END IF;
					IF start_now='1' THEN
						micro_state <= ide1;
						IF wsg='0' THEN --OR idea /="111" THEN
							IDEwait <= "0010000";
						ELSE
--							IDEwait <= "111111";		--slower access to status 
							IDEwait <= "0011000";		--slower access to status 
						END IF;
						ide_css0 <= '0';
						ide_wr <= IO_wr;
						ide_rd <= IO_rd;
					ELSE	
						micro_state <= ide7;
						IOcpuena <= '1';
					END IF;
				ELSIF ride_select='1' THEN
					IF (raddr(2 downto 0)/="000" OR highbyte='0') THEN --AND rena_in='1' THEN
						highbyte <= '0';
						IDEwr_data <= rcpudata_in;
					END IF;
					IF rstart_now='1' THEN
						micro_state <= ide1;
--							IDEwait <= "111111";		--slower access to status 
							IDEwait <= "0010100";		--slower access to status 
						ide_css0 <= '0';
						ide_wr <= rIO_wr;
						ide_rd <= rIO_rd;
					ELSE	
						micro_state <= ide7;
						rIOcpuena <= '1';
					END IF;
				END IF;
			WHEN ide1 =>
				IF selide='1' THEN
					ide_wr <= IO_wr;
					ide_rd <= IO_rd;
				ELSE	
					ide_wr <= rIO_wr;
					ide_rd <= rIO_rd;
				END IF;	

				IF IDEwait="0000000" THEN
					micro_state <= ide6;
					IDEwait <= "0000100";		--slower access to status 
					ide_wr <= '1';
					ide_rd <= '1';
				ELSE
					micro_state <= ide1;
					IDEwait <= IDEwait-1;
				END IF;
			WHEN ide6 =>
				IF IDEwait="0000000" THEN
					micro_state <= ide7;
					ide_wr <= '1';
					ide_rd <= '1';
					IDErd_data <= idedata_in;
					IF selide='1' THEN
						IOcpuena <= '1';
					ELSE	
						rIOcpuena <= '1';
					END IF;	
				ELSE
					micro_state <= ide6;
					IDEwait <= IDEwait-1;
				END IF;
			WHEN ide7 => 
				IF selide='1' THEN
					IF zclkena='0' THEN
						IOcpuena <= '1';
						micro_state <= ide7;
					ELSIF addr(2 downto 0)="000" THEN
						highbyte <= NOT highbyte;
					END IF;
				ELSE	
					IF rena_in='0' THEN
						rIOcpuena <= '1';
						micro_state <= ide7;
					ELSIF raddr(2 downto 0)="000" THEN
						highbyte <= NOT highbyte;
					END IF;
				END IF;	
				ide_csp0 <= '1';
				ide_css0 <= '1';
			WHEN OTHERS => null;
--				micro_state <= idle;
		END CASE;
	end if;
end process; 

-----------------------------------------------------------------
-- Simple UART only TxD
-----------------------------------------------------------------
TxD <= not shiftout;
process(n_reset, cpuclk, shift)
begin
	if shift="0000000000" then
		txbusy <= '0';
	else
		txbusy <= '1';
	end if;

	if n_reset='0' then
		shiftout <= '0';
		shift <= "0000000000"; 
	elsif cpuclk'event and cpuclk = '1' then
		IF RS232_select='1' AND rIO_wr='0' AND txbusy='0' THEN	--OUT FC07
			shift <=  '1' & rcpudata_in & '0';			--STOP,MSB...LSB, START
		end if;
		if clkgen/=0 then
			clkgen <= clkgen-1;
		else	
			clkgen <= "0110100000";--417-1;		--48MHz/115200
			shiftout <= not shift(0) and txbusy;
		   	shift <=  '0' & shift(9 downto 1);
		end if;
	end if;
end process; 

end;  
