------------------------------------------------------------------------------
------------------------------------------------------------------------------
--                                                                          --
-- Copyright (c) 2009 Tobias Gubener                                        -- 
-- Subdesign CPC T-REX by TobiFlex                                          --
--                                                                          --
-- This source file is free software: you can redistribute it and/or modify --
-- it under the terms of the GNU General Public License as published        --
-- by the Free Software Foundation, either version 3 of the License, or     --
-- (at your option) any later version.                                      --
--                                                                          --
-- This source file is distributed in the hope that it will be useful,      --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of           --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            --
-- GNU General Public License for more details.                             --
--                                                                          --
-- You should have received a copy of the GNU General Public License        --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.    --
--                                                                          --
------------------------------------------------------------------------------
------------------------------------------------------------------------------


LIBRARY ieee;
USE ieee.std_logic_1164.all; 


ENTITY cpcsymb IS 
	PORT
	(
		sysclk :  IN  STD_LOGIC;
		zdata :  IN  STD_LOGIC_VECTOR(7 DOWNTO 0);
		cpuclk :  IN  STD_LOGIC;
		cpcrst :  IN  STD_LOGIC;
		clkena_in :  IN  STD_LOGIC;
		crt_reset :  IN  STD_LOGIC;
		zwait :  IN  STD_LOGIC;
		vdata :  IN  STD_LOGIC_VECTOR(7 DOWNTO 0);
		IO_in :  IN  STD_LOGIC_VECTOR(7 DOWNTO 0);
		n_reset :  IN  STD_LOGIC;
		kbc :  IN  STD_LOGIC;
		kbd :  IN  STD_LOGIC;
		Joya :  IN  STD_LOGIC_VECTOR(5 DOWNTO 0);
		Joyb :  IN  STD_LOGIC_VECTOR(5 DOWNTO 0);
		crtclk :  IN  STD_LOGIC;
		msci :  IN  STD_LOGIC;
		msdi :  IN  STD_LOGIC;
		enable :  IN  STD_LOGIC;
		
		data_write :  OUT  STD_LOGIC_VECTOR(7 DOWNTO 0);
		vaddr :  OUT  STD_LOGIC_VECTOR(23 DOWNTO 0);
		sdaddr :  OUT  STD_LOGIC_VECTOR(23 DOWNTO 0);
		pixclk :  OUT  STD_LOGIC;
		Hsync :  OUT  STD_LOGIC;
		Vsync :  OUT  STD_LOGIC;
		Video :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
		io_wr :  BUFFER  STD_LOGIC;
		io_rd :  BUFFER  STD_LOGIC;
		mem_wr :  BUFFER  STD_LOGIC;
		mem_rd :  OUT  STD_LOGIC;
		state :  OUT  STD_LOGIC_VECTOR(1 DOWNTO 0);
		clkena :  BUFFER  STD_LOGIC;
		rdata :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
		ldata :  OUT  STD_LOGIC_VECTOR(15 DOWNTO 0);
		cpcreset :  OUT  STD_LOGIC;
		speed :  OUT  STD_LOGIC;
		pal :  OUT  STD_LOGIC;
		wsg :  BUFFER  STD_LOGIC;
		menue :  OUT  STD_LOGIC
	);
END cpcsymb;

ARCHITECTURE bdf_type OF cpcsymb IS 

COMPONENT pio
	PORT(cs : IN STD_LOGIC;
		 iowr : IN STD_LOGIC;
		 iord : IN STD_LOGIC;
		 cpuclk : IN STD_LOGIC;
		 addr : IN STD_LOGIC_VECTOR(1 DOWNTO 0);
		 datain : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 PAI : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 PBI : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 DO : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		 PAO : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		 PCO : OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
	);
END COMPONENT;

COMPONENT ay8912
	PORT(cpuclk : IN STD_LOGIC;
		 reset : IN STD_LOGIC;
		 cs : IN STD_LOGIC;
		 bc0 : IN STD_LOGIC;
		 bdir : IN STD_LOGIC;
		 Data_in : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 PortAin : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 chanA : OUT STD_LOGIC_VECTOR(10 DOWNTO 0);
		 chanB : OUT STD_LOGIC_VECTOR(10 DOWNTO 0);
		 chanC : OUT STD_LOGIC_VECTOR(10 DOWNTO 0);
		 links : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		 oData : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		 rechts : OUT STD_LOGIC_VECTOR(15 DOWNTO 0)
	);
END COMPONENT;

COMPONENT kbscan
	PORT(EPclk : IN STD_LOGIC;
		 nreset_in : IN STD_LOGIC;
		 kb_clk : IN STD_LOGIC;
		 kb_data : IN STD_LOGIC;
		 n_scread : IN STD_LOGIC;
		 enable : IN STD_LOGIC;
		 Joya : IN STD_LOGIC_VECTOR(5 DOWNTO 0);
		 Joyb : IN STD_LOGIC_VECTOR(5 DOWNTO 0);
		 sline : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
		 nreset : OUT STD_LOGIC;
		 speed : OUT STD_LOGIC;
		 pal : OUT STD_LOGIC;
		 wsg : OUT STD_LOGIC;
		 n_ready : OUT STD_LOGIC;
		 extended : OUT STD_LOGIC;
		 menue : OUT STD_LOGIC;
		 matrix : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		 scancode : OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
	);
END COMPONENT;

COMPONENT cpcglue
	PORT(n_reset : IN STD_LOGIC;
		 crt_reset : IN STD_LOGIC;
		 sysclk : IN STD_LOGIC;
		 cpuclk : IN STD_LOGIC;
		 clkena : IN STD_LOGIC;
		 n_iowr : IN STD_LOGIC;
		 flat_mem : IN STD_LOGIC;
		 intack : IN STD_LOGIC;
		 mem_wr : IN STD_LOGIC;
		 pal : IN STD_LOGIC;
		 crtclk : IN STD_LOGIC;
		 enable : IN STD_LOGIC;
		 addr : IN STD_LOGIC_VECTOR(23 DOWNTO 0);
		 data : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 vdata : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 Vsync : OUT STD_LOGIC;
		 int : OUT STD_LOGIC;
		 pixclk : OUT STD_LOGIC;
		 CPCHsync : OUT STD_LOGIC;
		 CPCVsync : OUT STD_LOGIC;
		 CPCVideo : OUT STD_LOGIC_VECTOR(15 DOWNTO 0);
		 sdaddr : OUT STD_LOGIC_VECTOR(23 DOWNTO 0);
		 vaddr : OUT STD_LOGIC_VECTOR(23 DOWNTO 0)
	);
END COMPONENT;

COMPONENT z80_timing
	PORT(clk : IN STD_LOGIC;
		 reset : IN STD_LOGIC;
		 int_in : IN STD_LOGIC;
		 clkena_in : IN STD_LOGIC;
		 n_wait : IN STD_LOGIC;
		 wsg : IN STD_LOGIC;
		 data_in : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 i_state : IN STD_LOGIC_VECTOR(10 DOWNTO 0);
		 int_out : OUT STD_LOGIC;
		 clkena : OUT STD_LOGIC;
		 n_wait_out : OUT STD_LOGIC;
		 data_out : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		 o_extra_cycle : OUT STD_LOGIC_VECTOR(2 DOWNTO 0)
	);
END COMPONENT;

COMPONENT tg80_fast
	PORT(clk : IN STD_LOGIC;
		 reset : IN STD_LOGIC;
		 interrupt : IN STD_LOGIC;
		 clkena_in : IN STD_LOGIC;
		 data_in : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 IO_in : IN STD_LOGIC_VECTOR(7 DOWNTO 0);
		 io_wr : OUT STD_LOGIC;
		 io_rd : OUT STD_LOGIC;
		 int_ack : OUT STD_LOGIC;
		 mem_wr : OUT STD_LOGIC;
		 mem_ce : OUT STD_LOGIC;
		 flat_mem : OUT STD_LOGIC;
		 address : OUT STD_LOGIC_VECTOR(23 DOWNTO 0);
		 data_write : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
		 i_state : OUT STD_LOGIC_VECTOR(10 DOWNTO 0);
		 state : OUT STD_LOGIC_VECTOR(1 DOWNTO 0)
	);
END COMPONENT;

COMPONENT ps2mouse
	PORT(Rst_n : IN STD_LOGIC;
		 Clk : IN STD_LOGIC;
		 T1MHz : IN STD_LOGIC;
		 cpuclk : IN STD_LOGIC;
		 iord : IN STD_LOGIC;
		 PS2_Clk_in : IN STD_LOGIC;
		 PS2_Data_in : IN STD_LOGIC;
		 enable : IN STD_LOGIC;
		 addr : IN STD_LOGIC_VECTOR(15 DOWNTO 0);
		 PS2_Clk : OUT STD_LOGIC;
		 PS2_Data : OUT STD_LOGIC;
		 wmouse : OUT STD_LOGIC;
		 DO : OUT STD_LOGIC_VECTOR(7 DOWNTO 0)
	);
END COMPONENT;

SIGNAL	addr :  STD_LOGIC_VECTOR(23 DOWNTO 0);
SIGNAL	PBI :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	PCO :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	DO :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	flat_mem :  STD_LOGIC;
SIGNAL	i_state :  STD_LOGIC_VECTOR(10 DOWNTO 0);
SIGNAL	int :  STD_LOGIC;
SIGNAL	intack :  STD_LOGIC;
SIGNAL	into :  STD_LOGIC;
SIGNAL	matrix :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	P2do :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	PAI :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	PAO :  STD_LOGIC_VECTOR(7 DOWNTO 0);
SIGNAL	PDO :  STD_LOGIC_VECTOR(7 DOWNTO 0);


BEGIN 

pio1 : pio
PORT MAP(cs => addr(11),
		 iowr => io_wr,
		 iord => io_rd,
		 cpuclk => cpuclk,
		 addr => addr(9 DOWNTO 8),
		 datain => DO,
		 PAI => PAI,
		 PBI => PBI,
		 DO => PDO,
		 PAO => PAO,
		 PCO => PCO);



sound : ay8912
PORT MAP(cpuclk => cpuclk,
		 reset => cpcrst,
		 cs => PCO(7),
		 bc0 => PCO(6),
		 bdir => PCO(7),
		 Data_in => PAO,
		 PortAin => matrix,
		 links => ldata,
		 oData => PAI,
		 rechts => rdata);


kb : kbscan
PORT MAP(EPclk => cpuclk,
		 nreset_in => n_reset,
		 kb_clk => kbc,
		 kb_data => kbd,
		 n_scread => '1',
		 enable => enable,
		 Joya => Joya,
		 Joyb => Joyb,
		 sline => PCO(3 DOWNTO 0),
		 nreset => cpcreset,
		 speed => speed,
		 pal => PBI(4),
		 wsg => wsg,
		 menue => menue,
		 matrix => matrix);


cpc : cpcglue
PORT MAP(n_reset => cpcrst,
		 crt_reset => crt_reset,
		 sysclk => sysclk,
		 cpuclk => cpuclk,
		 clkena => clkena,
		 n_iowr => io_wr,
		 flat_mem => flat_mem,
		 intack => intack,
		 mem_wr => mem_wr,
		 pal => PBI(4),
		 crtclk => crtclk,
		 enable => enable,
		 addr => addr,
		 data => DO,
		 vdata => vdata,
		 Vsync => PBI(0),
		 int => int,
		 pixclk => pixclk,
		 CPCHsync => Hsync,
		 CPCVsync => Vsync,
		 CPCVideo => Video,
		 sdaddr => sdaddr,
		 vaddr => vaddr);




cpc_timing : z80_timing
PORT MAP(clk => cpuclk,
		 reset => cpcrst,
		 int_in => int,
		 clkena_in => clkena_in,
		 n_wait => zwait,
		 data_in => "11111111",
		 wsg => wsg,
		 i_state => i_state,
		 int_out => into,
		 clkena => clkena);


cpu : tg80_fast
PORT MAP(clk => cpuclk,
		 reset => cpcrst,
		 interrupt => into,
		 clkena_in => clkena,
		 data_in => zdata,
		 IO_in => IO_in OR P2do OR PDO,
		 io_wr => io_wr,
		 io_rd => io_rd,
		 int_ack => intack,
		 mem_wr => mem_wr,
		 mem_ce => mem_rd,
		 flat_mem => flat_mem,
		 address => addr,
		 data_write => DO,
		 i_state => i_state,
		 state => state);


mouse : ps2mouse
PORT MAP(Rst_n => cpcrst,
		 Clk => cpuclk,
		 T1MHz => crtclk,
		 cpuclk => cpuclk,
		 iord => io_rd,
		 PS2_Clk_in => msci,
		 PS2_Data_in => msdi,
		 enable => enable,
		 addr => addr(15 DOWNTO 0),
		 DO => P2do);

pal <= PBI(4);
data_write <= DO;

END bdf_type;