------------------------------------------------------------------------------
------------------------------------------------------------------------------
--                                                                          --
-- Copyright (c) 2005-2009 Tobias Gubener                                   -- 
-- Subdesign CPC T-REX by TobiFlex                                          --
--                                                                          --
-- This source file is free software: you can redistribute it and/or modify --
-- it under the terms of the GNU General Public License as published        --
-- by the Free Software Foundation, either version 3 of the License, or     --
-- (at your option) any later version.                                      --
--                                                                          --
-- This source file is distributed in the hope that it will be useful,      --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of           --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            --
-- GNU General Public License for more details.                             --
--                                                                          --
-- You should have received a copy of the GNU General Public License        --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.    --
--                                                                          --
------------------------------------------------------------------------------
------------------------------------------------------------------------------


library IEEE;
use IEEE.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity crtc is
    port (
	MA     : out STD_LOGIC_VECTOR (13 downto 0);
	RA     : out STD_LOGIC_VECTOR (4 downto 0);
	HSYNC  : out STD_LOGIC;
	VSYNC  : out STD_LOGIC;
	DISP   : buffer STD_LOGIC;
	CPCHSYNC  : out STD_LOGIC;
	CPCVSYNC  : out STD_LOGIC;
	CPCDISP   : out STD_LOGIC;
	CPCBLACK  : out STD_LOGIC;
	IOCLK  : in STD_LOGIC;
	RS     : in STD_LOGIC;
	CS     : in STD_LOGIC;
	RW     : in STD_LOGIC;
	Din    : in STD_LOGIC_VECTOR (7 downto 0);
	RESET  : in STD_LOGIC;
	CLK    : in STD_LOGIC;
	PAL    : in STD_LOGIC
    );
end crtc;

architecture logic of crtc is

-- 6845 registers
signal RHtot		: STD_LOGIC_VECTOR (7 downto 0);
signal RHdisp		: STD_LOGIC_VECTOR (7 downto 0);
signal RHsyncpos	: STD_LOGIC_VECTOR (7 downto 0);
signal RHwidth		: STD_LOGIC_VECTOR (3 downto 0);
signal RVwidth		: STD_LOGIC_VECTOR (5 downto 0);
signal Vtot			: STD_LOGIC_VECTOR (6 downto 0);
signal RVtot		: STD_LOGIC_VECTOR (6 downto 0);
signal RVtaj		: STD_LOGIC_VECTOR (4 downto 0);
signal RRVtaj		: STD_LOGIC_VECTOR (4 downto 0);
signal RVdisp		: STD_LOGIC_VECTOR (6 downto 0);
signal RVsyncpos	: STD_LOGIC_VECTOR (6 downto 0);
signal Rmax			: STD_LOGIC_VECTOR (4 downto 0);
signal RRmax		: STD_LOGIC_VECTOR (4 downto 0);
signal RAdrH		: STD_LOGIC_VECTOR (5 downto 0);
signal RAdrL		: STD_LOGIC_VECTOR (7 downto 0);
signal RRadr		: STD_LOGIC_VECTOR (4 downto 0);

--------------------
signal cntH			: STD_LOGIC_VECTOR (7 downto 0);
signal cntHP		: STD_LOGIC_VECTOR (7 downto 0);
signal cntV			: STD_LOGIC_VECTOR (6 downto 0);
signal cntVP		: STD_LOGIC_VECTOR (6 downto 0);
signal cntRA		: STD_LOGIC_VECTOR (4 downto 0);
signal cntMA		: STD_LOGIC_VECTOR (13 downto 0);
signal cntHS		: STD_LOGIC_VECTOR (3 downto 0);
signal cntVS		: STD_LOGIC_VECTOR (5 downto 0);
signal cnttaj		: STD_LOGIC_VECTOR (4 downto 0);
signal RMA			: STD_LOGIC_VECTOR (13 downto 0);
signal enaHSS		: STD_LOGIC;
signal enaHSC		: STD_LOGIC;
signal enaHtot		: STD_LOGIC;
signal enaVtot		: STD_LOGIC;
signal enaV			: STD_LOGIC;
signal enaRA		: STD_LOGIC;
signal split		: STD_LOGIC;
signal Vdisp		: STD_LOGIC;
signal nodisp		: STD_LOGIC;
signal PALD			: STD_LOGIC;

begin
disp <= '1' when (cntV < RVdisp) and (cntH < RHdisp) and nodisp='0' else '0';
cpcblack <= '1' when cntVS < 27 else '0';
-------------------------------------------------------------------------
--H Line, H Sync and MA
-------------------------------------------------------------------------
process (clk, reset)
begin 
	if reset='0' then 
		cntH <= (others=>'0');
		cntMA <= (others=>'0');
		Hsync <= '0';
	elsif rising_edge(clk) then
		CPCdisp <= disp;
		if enaHtot='1' then
			cntH <= (others=>'0');
			if split='1' then
				cntMA<=RAdrH&RAdrL;
				RMA<=RAdrH&RAdrL;
			elsif enaRA='1' then
				RMA <= RMA+("00000"&RHdisp);
				cntMA <= RMA+("00000"&RHdisp);
			else
				cntMA<=RMA;
			end if;
		else
			cntH <= cntHP;

			cntMA <= cntMA + 1;
		end if;		
		
		if cntHS=RHwidth then
			Hsync<='0';
			cntHS <= B"0000";
			
		elsif cntH=RHsyncpos then
--			cntHS<=B"0001";
			cntHS<=B"0010";
			
			Hsync<='1';
		else
			if cntHS/=0 then
				cntHS<=cntHS+1;	
			end if;	
		end if;
		
		if cntHS=9 or cntHS=RHwidth then
			CPCHsync<='0';
		elsif cntHS=3 then
			CPCHsync<='1';
		end if;
	end if;
end process;
MA<=cntMA;
cntHP <= cntH+1;
enaHtot<='1' when cntH=RHtot else '0';

-------------------------------------------------------------------------
--Row
-------------------------------------------------------------------------
process (clk, reset)
begin 
	if reset='0' then 
		cntRA <= (others=>'0');
	elsif rising_edge(clk) then
		if enaHtot='1' then
			Rmax <= RRmax;
			RVtaj <= RRVtaj;
			if enaRA='1' or cnttaj=1 then
				cntRA <= (others=>'0');
			else
				cntRA <= cntRA+1;
			end if;		
		end if;	
	end if;
end process;
RA<=cntRA;
enaRA<='1' when cntRA=Rmax or cnttaj=1 or Rmax=0 else '0';

-------------------------------------------------------------------------
--V Line + Adjust
-------------------------------------------------------------------------
process (clk, reset, cntV)
begin 
	cntVP <= cntV+1;
	if reset='0' then 
		cntV <= (others=>'0');
	elsif rising_edge(clk) then
		if enaHtot='1' then
			Vtot <= RVtot;
			if split='1' then 
				cntV <= (others=>'0');
			else
				if enaRA='1' then 
					cntV <= cntVP;
				end if;			
			end if;		
			if enaVtot='1' and enaRA='1' then 
				cnttaj <= RVtaj;
			elsif cnttaj/=0 then
				cnttaj <= cnttaj-1;
			end if;			
		end if;	
	end if;
end process;
split<='1' when (enaVtot='1' and enaRA='1' and RVtaj=0) or cnttaj=1 else '0';
enaVtot<='1' when cntV = Vtot or Vtot=0 else '0';

-------------------------------------------------------------------------
--V SYNC
-------------------------------------------------------------------------
process (clk, reset)
begin 
	if reset='0' then 
		cntVS <= (others=>'0');
		Vsync <= '0';
	elsif rising_edge(clk) then
		if enaHtot='1' then
			if (cntVP=RVsyncpos and enaRA='1') OR (RVsyncpos=0 AND split='1')then
				Vsync<='1';
				cntVS <= B"000001";
			else
				if cntVS /= 63 then
					cntVS <= cntVS+1;
				end if;
			end if;			
			if cntVS=RVwidth then
				Vsync<='0';
			end if;
			if cntVS=2 then
				CPCVsync<='1';
			end if;
			if cntVS=6 or cntVS=RVwidth then
				CPCVsync<='0';
			end if;
		end if;
	end if;
end process;
-------------------------------------------------------------------------
--IO Regs
-------------------------------------------------------------------------
process (ioclk, reset)
begin 
	if reset='0' then 
		RHtot<="00111111";
		RHdisp<="00101000";
		RHsyncpos<="00101110";
		RHwidth<="1110";
		RVwidth<="001000";
		RVtot<="0100110";
		RRVtaj<="00000";
		RVdisp<="0011001";
		nodisp<='0';
		RVsyncpos<="0011110";
		RRmax<="00111";
		RAdrH<="110000";
		RAdrL<="00000000";
	elsif rising_edge(ioclk) then
		PALD <= PAL;
		IF PALD='0' AND PAL='1' THEN
			RVtot<="0100110";
			RRVtaj<="00000";
			RVsyncpos<="0011110";
		END IF;
		IF PALD='1' AND PAL='0' THEN
			RVtot<="0011111";
			RRVtaj<="00110";
			RVsyncpos<="0011011";
		END IF;
		if cs='0' and rw='0' then	
			if rs='0' then
				RRadr <= Din(4 downto 0);
			else
				if RRadr="00000" then
					RHtot<=Din;
				end if;
				if RRadr="00001" then
					RHdisp<=Din;
				end if;
				if RRadr="00010" then
					RHsyncpos<=Din;
				end if;
				if RRadr="00011" then
					if Din(7 downto 4)="0000" then
						RVwidth<="01"& Din(7 downto 4);
					else
						RVwidth<="00"& Din(7 downto 4);
					end if;
--					if Din(3 downto 0)="0000" then
--						RHwidth<='1'& Din(3 downto 0);
--					else
--						RHwidth<='0'& Din(3 downto 0);
--					end if;
					RHwidth <= Din(3 downto 0);
				end if;
				if RRadr="00100" then
					RVtot<=Din(6 downto 0);
				end if;
				if RRadr="00101" then
					RRVtaj<=Din(4 downto 0);
				end if;
				if RRadr="00110" then
					RVdisp<=Din(6 downto 0);
				end if;
				if RRadr="00111" then
					RVsyncpos<=Din(6 downto 0);
				end if;
				if RRadr="01000" then
					IF Din(5 downto 4)="11" THEN				
						nodisp <= '1';
					ELSE
						nodisp <= '0';
					END IF;	
				end if;
				if RRadr="01001" then
					RRmax<=Din(4 downto 0);
				end if;
				if RRadr="01100" then
					RAdrH<=Din(5 downto 0);
				end if;
				if RRadr="01101" then
					RAdrL<=Din;
				end if;
			end if;
		end if;
	end if;	
end process;
end logic;

