------------------------------------------------------------------------------
------------------------------------------------------------------------------
--                                                                          --
-- Copyright (c) 2008-2009 Tobias Gubener                                   -- 
-- Subdesign CPC T-REX by TobiFlex                                          --
--                                                                          --
-- This source file is free software: you can redistribute it and/or modify --
-- it under the terms of the GNU General Public License as published        --
-- by the Free Software Foundation, either version 3 of the License, or     --
-- (at your option) any later version.                                      --
--                                                                          --
-- This source file is distributed in the hope that it will be useful,      --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of           --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            --
-- GNU General Public License for more details.                             --
--                                                                          --
-- You should have received a copy of the GNU General Public License        --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.    --
--                                                                          --
------------------------------------------------------------------------------
------------------------------------------------------------------------------


library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.STD_LOGIC_UNSIGNED.all;


entity cyclone is
   port ( 
	sysclk: in std_logic;	
	addon_out: out std_logic_vector(8 downto 0);		
	addonin: in std_logic_vector(5 downto 0);		
	Ring : in std_logic_vector(3 downto 0);
	IOB_clk: in std_logic;
	
	idedatai: in std_logic_vector(15 downto 0);		
	idedatao: out std_logic_vector(15 downto 0);		
	idea: in std_logic_vector(2 downto 0);		
	ide_css0: in std_logic;			
	ide_csp1: in std_logic;
	ide_wr: in std_logic;			
	ide_rd: in std_logic;			
	ide_csp0: in std_logic;
	TxD: in std_logic;
	ide1irq: out std_logic;	
	kb_clk: out std_logic;	
	kb_data: out std_logic;	
	ms_clk: out std_logic;	
	ms_data: out std_logic;	
	kb_clki: in std_logic;	
	kb_datai: in std_logic;	
	ms_clki: in std_logic;	
	ms_datai: in std_logic;	
	JoyA: out std_logic_vector(5 downto 0);		
	JoyB: out std_logic_vector(5 downto 0);		
	right: in std_logic_vector(15 downto 0);		
	left: in std_logic_vector(15 downto 0);		
	target: in std_logic;	
	sright: in std_logic_vector(15 downto 0);		
	sleft: in std_logic_vector(15 downto 0)		

  );

end cyclone;


architecture wire of cyclone is

	signal idedatad: std_logic_vector(14 downto 0);
	signal dmuxd: std_logic_vector(14 downto 0);
	signal dmux: std_logic_vector(15 downto 0);		
	signal C2MHz: std_logic;
	signal shift: std_logic_vector(16 downto 0);		
	signal lrclk: std_logic;
	signal count: std_logic_vector(4 downto 0);		
	signal jshift: std_logic_vector(15 downto 0);
	signal ioshift: std_logic;
	signal ide_wrd: std_logic;			
	signal sync: std_logic;			
	signal cs0d: std_logic;			
	signal cs1d: std_logic;			
	signal addonout: std_logic_vector(8 downto 0);		


-------------------------------------------------------------------	
--  TRANSMITTER		

--addonout	0			1			2			3     
--(0)								TxD		 	sync
--(1)		ide_rd 					ide_csp0	kb/ms_datai
--(2)		ide_wr					ide_css0 	kb/ms_clki	

--(3)		   IOB_clk
--(4)		idea(0)		idei(0)		idei(5)		idei(10)	
--(5)		idea(1) 	idei(1)		idei(6)		idei(11)
--(6)		idea(2) 	idei(2)		idei(7)		idei(12)
--(7)		idea(3) 	idei(3)		idei(8)		idei(13)
--(8)		idei(15)	idei(4)		idei(9)		idei(14)

BEGIN
	addon_out(3) <= IOB_clk;
	PROCESS(sysclk)
	BEGIN
		IF sysclk'event AND sysclk = '0' THEN
			addon_out(0) <= addonout(0);
			addon_out(1) <= addonout(1);
			addon_out(2) <= addonout(2);
			addon_out(4) <= addonout(4);
			addon_out(5) <= addonout(5);
			addon_out(6) <= addonout(6);
			addon_out(7) <= addonout(7);
			addon_out(8) <= addonout(8);
		END IF;
	END PROCESS; 
	
	dmux <= idedatai WHEN ide_wrd='1' AND ide_wr='0'  ELSE "00000000000000"&lrclk&shift(16);
	PROCESS(IOB_clk)
	BEGIN
		IF IOB_clk'event AND IOB_clk = '1' THEN
			addonout(0) <= '0';
			IF Ring(0)='1' THEN
				ide_wrd <= ide_wr;
				cs0d <= ide_csp0;
				cs1d <= ide_css0;
				addonout(1) <= ide_rd;
				addonout(2) <= ide_wr;

				addonout(7 downto 4) <= '0'&idea;
				addonout(8) <= dmux(15);
				dmuxd <= dmux(14 downto 0);
			END IF;
			IF Ring(1)='1' THEN
				addonout(8 downto 4) <= dmuxd(4 downto 0);
			END IF;
			IF Ring(2)='1' THEN
				addonout(0) <= TxD;
				addonout(1) <= cs0d;
				addonout(2) <= cs1d;

				addonout(8 downto 4) <= dmuxd(9 downto 5);
			END IF;
			IF Ring(3)='1' THEN
				addonout(0) <= sync;			--Syncbit
				sync <= NOT sync;
				IF sync='0' THEN
					addonout(1) <= ms_datai;
					addonout(2) <= ms_clki;
				ELSE
					addonout(1) <= kb_datai;
					addonout(2) <= kb_clki;
				END IF;

				addonout(8 downto 4) <= dmuxd(14 downto 10);
			END IF;
		END IF;
	END PROCESS; 
	
-------------------------------------------------------------------	
--  RESEIVER		
--			0		1		2			3     

--addonin	1		2		3			0
--(0)		idei(0)	idei(5)	idei(10)	idei(15)	
--(1)		idei(1)	idei(6)	idei(11)	ide1irq	
--(2)		idei(2)	idei(7)	idei(12)	kb_clk
--(3)		idei(3)	idei(8)	idei(13)	kb_data
--(4)		idei(4)	idei(9)	idei(14)	ms_clk
--(5)		C2MHz	ioshift				ms_data
	PROCESS(IOB_clk)
	BEGIN
		IF IOB_clk'event AND IOB_clk = '1' THEN
			IF Ring(0)='1' THEN
				idedatao(14 downto 0) <= idedatad;
				idedatao(15) <= addonin(0);
				ide1irq <= addonin(1);
				kb_clk <= addonin(2);
				kb_data <= addonin(3);
				ms_clk <= addonin(4);
				ms_data <= addonin(5);
			END IF;
			IF Ring(1)='1' THEN
				idedatad(4 downto 0) <= addonin(4 downto 0);
				C2MHz <= addonin(5);
			END IF;
			IF Ring(2)='1' THEN
				idedatad(9 downto 5) <= addonin(4 downto 0);
				ioshift <= addonin(5);
			END IF;
			IF Ring(3)='1' THEN
				idedatad(14 downto 10) <= addonin(4 downto 0);
			END IF;
		END IF;
	END PROCESS; 

-------------------------------------------------------------------	
--  SOUND		
--Audioshifter
	
	PROCESS(C2MHz)
	BEGIN
		IF C2MHz'event AND C2MHz = '0' THEN
			IF count=0 then	
				count <= "10111";--23
				lrclk <= NOT lrclk;				
--				shift <= ("00111111111111111");			max output	
				IF lrclk='1' THEN
					IF target='1' THEN						
						shift <= ('0'&left);
					ELSE				
						shift <= ('0'&sleft);				
					END IF;
				ELSE
					IF target='1' THEN						
						shift <= ('0'&right);			
					ELSE				
						shift <= ('0'&sright);			
					END IF;
				END IF;
			ELSE
				count <= count-1;
				IF count<17 THEN
					shift <= (shift(15 downto 0)&'0');
				END IF;
			END IF;
		END IF;
	END PROCESS; 
	
-------------------------------------------------------------------	
--Joystick
	PROCESS(C2MHz)
	BEGIN
		IF C2MHz'event AND C2MHz = '1' THEN
			IF jshift(15)='0' THEN
				jshift <=("111111111111111"&ioshift);
--				lpt_bsy=NOT jshift(14);
				JoyA<=('1'&jshift(8)&jshift(11 downto 9)&jshift(7));
				JoyB<=('1'&jshift(5)&jshift(2)&jshift(3)&jshift(4)&jshift(6));
--				JoyA<=('1'&jshift(8)&jshift(7)&jshift(9)&jshift(10)&jshift(11));
--				JoyB<=('1'&jshift(5)&jshift(6)&jshift(4)&jshift(3)&jshift(2));
			ELSE
				jshift<=(jshift(14 downto 0)&ioshift);
			END IF;
		END IF;
	END PROCESS; 

END;  
