------------------------------------------------------------------------------
------------------------------------------------------------------------------
--                                                                          --
-- Copyright (c) 2009 Tobias Gubener                                        -- 
-- Subdesign CPC T-REX by TobiFlex                                          --
--                                                                          --
-- This source file is free software: you can redistribute it and/or modify --
-- it under the terms of the GNU General Public License as published        --
-- by the Free Software Foundation, either version 3 of the License, or     --
-- (at your option) any later version.                                      --
--                                                                          --
-- This source file is distributed in the hope that it will be useful,      --
-- but WITHOUT ANY WARRANTY; without even the implied warranty of           --
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            --
-- GNU General Public License for more details.                             --
--                                                                          --
-- You should have received a copy of the GNU General Public License        --
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.    --
--                                                                          --
------------------------------------------------------------------------------
------------------------------------------------------------------------------


library IEEE;
use IEEE.std_logic_1164.all;
--use ieee.std_logic_unsigned.all;

entity pio is
    port (
	addr			: in STD_LOGIC_VECTOR (1 downto 0);
	datain			: in STD_LOGIC_VECTOR (7 downto 0);
	cs				: in STD_LOGIC;
	iowr			: in STD_LOGIC;
	iord			: in STD_LOGIC;
	cpuclk			: in STD_LOGIC;
	
	PBI				: in STD_LOGIC_VECTOR (7 downto 0);
	PAI				: in STD_LOGIC_VECTOR (7 downto 0);		--Keyboarddaten
	PAO     		: buffer STD_LOGIC_VECTOR (7 downto 0);		--sounddaten
	PCO     		: out STD_LOGIC_VECTOR (7 downto 0);		--tastatur und steuerung
	DO		     	: out STD_LOGIC_VECTOR (7 downto 0)
    );
end pio;

architecture logic of pio is
signal 	PAdir		: STD_LOGIC;
signal 	PCHdir		: STD_LOGIC;
signal 	PCO_d		: STD_LOGIC_VECTOR (7 downto 0);	



begin
process (cpuclk, PAO, PCO_d, PCHdir, PAdir, PBI, PAI, addr, cs, iord)
begin 

	PCO <= PCO_d;
	IF PCHdir='1' THEN
		PCO(7 downto 4) <= "1111";
	END IF;	
	
	IF rising_edge(cpuclk) THEN  
		IF cs='0' AND iowr='0' THEN
			IF addr(1 downto 0)="00" THEN
				PAO <= datain;
			END IF;
			IF addr(1 downto 0)="10" THEN
				PCO_d <= datain;
			END IF;
			IF addr(1 downto 0)="11" AND datain(7)='0' THEN
				CASE datain(3 downto 1) IS
					WHEN "000" => PCO_d(0) <= datain(0);
					WHEN "001" => PCO_d(1) <= datain(0);
					WHEN "010" => PCO_d(2) <= datain(0);
					WHEN "011" => PCO_d(3) <= datain(0);
					WHEN "100" => PCO_d(4) <= datain(0);
					WHEN "101" => PCO_d(5) <= datain(0);
					WHEN "110" => PCO_d(6) <= datain(0);
					WHEN OTHERS => PCO_d(7) <= datain(0);
				END CASE;	
			END IF;
			IF addr(1 downto 0)="11" AND datain(7)='1' THEN
				PCHdir <= datain(3);
				PAdir <= datain(4);
				PAO <= "00000000";
				PCO_d <= "00000000";
			END IF;
		END IF;
	END IF;
	
	DO <= "00000000";
	IF cs='0' AND iord='0' THEN
		IF addr(1 downto 0)="00" THEN	--Keyboarddaten
			IF PAdir='1' THEN
				DO <= PAI;
			ELSE
				DO <= PAO;
			END IF;	
		END IF;
		IF addr(1 downto 0)="01" THEN
			DO <= ("111"&PBI(4)&"111"&PBI(0));
		END IF;
		IF addr(1 downto 0)="10" THEN
			DO <= PCO_d;
		END IF;
	END IF;
END process;
end logic;
